﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Security.Principal;

using codejockey.Shared.Web;
using codejockey.Shared.XML;
using codejockey.Shared.Win32;

namespace codejockey.BingPaper
{
  public class LastImageRetrieved
  {
    public DateTime Retrieval
    {
      get;
      set;
    }

    public string ImageName
    {
      get;
      set;
    }

    public string LastStatus
    {
      get;
      set;
    }
  }

  public static class BingUtils
  {
    private const string TrackerFileName = "BingDLStatus.xml";
    private const string baseURL = "http://www.bing.com";
    
    private static string bingImageName = string.Empty;
    private static Dictionary<string, string> tokenList = new Dictionary<string, string>();

    public static Dictionary<string, string> TokenList
    {
      get 
      {
        if (tokenList.Count == 0)
          PopulateTokenList();

        return tokenList; 
      }
    }

    private static void PopulateTokenList()
    {
      tokenList.Add("$month$", ParseTokens("$month$"));
      tokenList.Add("$day$", ParseTokens("$day$"));
      tokenList.Add("$year$", ParseTokens("$year$"));
      tokenList.Add("$hour$", ParseTokens("$hour$"));
      tokenList.Add("$minute$", ParseTokens("$minute$"));
      tokenList.Add("$second$", ParseTokens("$second$"));
      tokenList.Add("$milisecond$", ParseTokens("$milisecond$"));
      tokenList.Add("$currentuser$", ParseTokens("$currentuser$"));
      tokenList.Add("$currentuseranddomain$", ParseTokens("$currentuserwithdomain$"));
      tokenList.Add("$bingimagename$", ParseTokens("$bingimagename$"));
    }

    public static string GetBingImageName(Uri url)
    {
      string result = string.Empty;

      string bingContent = WebUtils.GetContent(url);
      result = ParseContentForImageName(bingContent);

      return result;
    }

    public static string GetBingImageName(Uri url, bool scrubName)
    {
      string result = GetBingImageName(url);
      return ScrubBingString(result);
    }

    /// <summary>
    /// GetCurrentBingImage first downloads the web page and scans the content for the presence of
    /// the known moniker.  The string is then taken apart and the URL to the image file itself is
    /// determined and then retrieved and saved.
    /// </summary>
    /// <param name="url">The base URL.</param>
    /// <param name="tokenizedString">Folder to save the file to.</param>
    /// <returns>The file name the image was saved as.</returns>
    /// <remarks>There has to be a better way?  I've examined the Bing API and did not see
    /// anything in there about the ability to download the current image.</remarks>
    public static string GetCurrentBingImage(Uri url, string pathToSaveFileTo, string fileNameToSaveAs)
    {
      string fileName = string.Empty;

      ValidateParams(url, pathToSaveFileTo);

      if (!pathToSaveFileTo.EndsWith(@"\"))
        pathToSaveFileTo = pathToSaveFileTo + @"\";

      fileName = string.Format(@"{0}", pathToSaveFileTo);
      
      if (string.IsNullOrEmpty(fileNameToSaveAs))
        fileName = fileName + GetFileName();
      else
        fileName = fileName + fileNameToSaveAs;

      string sourceString = GetBingImageName(url);
      ScrubBingString(sourceString);
      
      CreateFilePath(ParseTokens(pathToSaveFileTo));
      fileName = ParseTokens(fileName);

      if (!fileName.Contains(".jpg"))
        fileName = fileName + ".jpg";

      bool download = IsDownloadNeeded(sourceString, fileName);
      
      string temp = string.Format("{0}/{1}", url.ToString(), sourceString);

      if (download)
      {
        try
        {
          WebUtils.GetFile(new Uri(temp), fileName);
        }
        catch (Exception)
        {
          throw;
        }
      }

      if (File.Exists(fileName))
        return fileName;
      else
        return string.Empty;
    }

    private static bool IsDownloadNeeded(string imageName, string fileName)
    {
      bool result = false;
      LastImageRetrieved imageStatus = null;

      string trackerFile = string.Format(@"{0}\{1}", Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), TrackerFileName);

      if (File.Exists(trackerFile))
        imageStatus = XMLUtils.DeserializeXmlFromFile(trackerFile, typeof(LastImageRetrieved)) as LastImageRetrieved;

      if (imageStatus != null && imageStatus.ImageName.CompareTo(imageName) == 0 && File.Exists(fileName))
      {
        imageStatus.LastStatus = string.Format("Image: {0} has already been downloaded.", imageName);
        result = false;
      }
      else
      {
        result = true;
        imageStatus = new LastImageRetrieved
        {
          ImageName = imageName,
          LastStatus = string.Format("Image: {0} newly downloaded.", imageName),
          Retrieval = DateTime.Now
        };
      }

      XMLUtils.SerializeXMLToFile(trackerFile, imageStatus);

      return result;
    }
    
    private static string ScrubBingString(string sourceString)
    {
      int index = sourceString.LastIndexOf("/");
      bingImageName = sourceString.Substring(index + 1);
      bingImageName = bingImageName.Replace(".jpg", "");

      return bingImageName;
    }

    public static string GetCurrentBingImage(string url, string pathToSaveFileTo, string fileNameToSaveAs)
    {
      return GetCurrentBingImage(new Uri(url), pathToSaveFileTo, fileNameToSaveAs);
    }

    public static string GetCurrentBingImage(Uri url, string pathToSaveFileTo)
    {
      return GetCurrentBingImage(url, pathToSaveFileTo, string.Empty);
    }

    public static string GetCurrentBingImage(string url, string pathToSaveFileTo)
    {
      return GetCurrentBingImage(new Uri(url), pathToSaveFileTo);
    }

    public static string GetCurrentBingImage(string pathToSaveFileTo)
    {
      return GetCurrentBingImage(new Uri(baseURL), pathToSaveFileTo);
    }

    /// <summary>
    /// Parses the name of the tokens from path.
    /// </summary>
    /// <param name="tokenizedString">The path to save file to.</param>
    /// <returns>A string with known tokens replaced.</returns>
    /// <remarks>Curernt valid tokens: $year$, $month$, $day$, $hour$, $minute$
    /// $second$, $milisecond$, $currentuser$, $currentuserwithdomain$, $bingimagename$</remarks>
    public static string ParseTokens(string tokenizedString)
    {
      tokenizedString = tokenizedString.Replace("$year$", DateTime.Now.Year.ToString());
      tokenizedString = tokenizedString.Replace("$month$", DateTime.Now.Month.ToString());
      tokenizedString = tokenizedString.Replace("$day$", DateTime.Now.Day.ToString());
      tokenizedString = tokenizedString.Replace("$hour$", DateTime.Now.Hour.ToString());
      tokenizedString = tokenizedString.Replace("$minute$", DateTime.Now.Minute.ToString());
      tokenizedString = tokenizedString.Replace("$second$", DateTime.Now.Second.ToString());
      tokenizedString = tokenizedString.Replace("$milisecond$", DateTime.Now.Millisecond.ToString());
      
      if (string.IsNullOrEmpty(bingImageName))
      {
        bingImageName = GetBingImageName(new Uri(baseURL));
        ScrubBingString(bingImageName);
      }

      tokenizedString = tokenizedString.Replace("$bingimagename$", bingImageName);

      if (tokenizedString.Contains("$currentuser$"))
      {
        WindowsIdentity id = WindowsIdentity.GetCurrent();
        int index = id.Name.IndexOf(@"\");
        string userName = id.Name.Substring(index + 1);
        tokenizedString = tokenizedString.Replace("$currentuser$", userName);
      }

      if (tokenizedString.Contains("$currentuserwithdomain$"))
      {
        WindowsIdentity id = WindowsIdentity.GetCurrent();
        tokenizedString = tokenizedString.Replace("$currentuserwithdomain$", id.Name);
      }

      if (tokenizedString.Contains("$currentdomain$"))
      {
        WindowsIdentity id = WindowsIdentity.GetCurrent();
        int index = id.Name.IndexOf(@"\");
        string domainName = id.Name.Substring(0, index);
        tokenizedString = tokenizedString.Replace("$currentdomain$", domainName);
      }

      return tokenizedString;
    }

    private static string ParseContentForImageName(string bingContent)
    {
      const string UnableToDetermineURLErrorMessage = "Unable to determine the URL that is necessary to download the current image.";
      const string ImageContainerSignatureStartToken = "g_img={url:'";
      const string ImageContainerSignatureEndToken = "'}";
      const string ElementDelimiter = @"'";

      int startIndex = bingContent.IndexOf(ImageContainerSignatureStartToken);
      int endIndex = 0;

      if (startIndex > 0)
        endIndex = bingContent.IndexOf(ImageContainerSignatureEndToken, startIndex);

      if (startIndex < 0 || endIndex < 0)
        throw new Exception(UnableToDetermineURLErrorMessage);

      string sourceString = bingContent.Substring(startIndex, endIndex - startIndex);

      startIndex = sourceString.IndexOf(ElementDelimiter);
      endIndex = sourceString.IndexOf(ElementDelimiter, startIndex + 1);

      if (startIndex == 0 || endIndex == 0)
        throw new Exception(UnableToDetermineURLErrorMessage);

      sourceString = sourceString.Substring(startIndex + 1, endIndex - startIndex - 1);

      return sourceString;
    }

    private static void CreateFilePath(string pathToSaveFileTo)
    {
      if (!Directory.Exists(pathToSaveFileTo))
        Directory.CreateDirectory(pathToSaveFileTo);
    }

    private static string GetFileName()
    {
      string tempFile = string.Format("{0:00}{1:00}{2}.jpg", DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Year);

      return tempFile;
    }

    private static void ValidateParams(Uri url, string pathToSaveFileTo)
    {
      if (url == null)
        throw new Exception("A valid Uri object must be provided.");

      if (string.IsNullOrEmpty(pathToSaveFileTo))
        throw new Exception("A valid path must be provided to save the file to.");
    }
  }
}
