﻿#region codejockey.net Copyright Banner
/***************************************************************\
 * Copyright 2010 codejockey.net.                                *
 *                                                               *
 * This code is property of codejockey.net and cannot be used    *
 * without explicit permission given by a codejockey.net         *
 * Representative.  For details contact: source@codejockey.net   *
 \***************************************************************/
#endregion

using System;
using System.Collections.Generic;
using System.Configuration;
using IO = System.IO;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Threading;

using codejockey.Shared.Win32;
using codejockey.Shared.Text;
using codejockey.Shared.Text.ReplacementTokens;

using codejockey.BingPaper.Utils;
using codejockey.BingPaper.Utils.ReplacementTokens;
using codejockey.BingPaper.Configuration;

using Microsoft.WindowsAPICodePack.Taskbar;

namespace codejockey.BingPaper
{
  /// <summary>
  /// Interaction logic for MainWindow.xaml
  /// </summary>
  public partial class MainWindow : Window
  {
    #region Default Values

    private const string appTitle = "BingPaper";
    private const string appTitleWithCountryCode = appTitle + " - [{0}]";
    private const string defaultBaseURL = "http://www.bing.com";
    private const string defaultRssURL = defaultBaseURL + "/HPImageArchive.aspx?format=xml&idx=0&n=1&cc=en-us";
    private const int defaultLengthLimit = 255;

    #endregion

    #region Configuration Keys

    const string ChangeWallPaperConfigKey = "ChangeWallpaper";
    const string BaseURLKey = "BaseURL";
    const string WallPaperStyleKey = "WallpaperStyle";
    const string ImageDumpFolderKey = "ImageDumpFolder";
    const string ImageNameKey = "ImageName";

    #endregion

    #region CommandLine Args

    const string setWallPaperArg = "setwallpaper";
    const string filePathArg = "filepath";
    const string fileNameArg = "filename";
    const string wallPaperStyleArg = "wallpaperstyle";
    const string helpArg = "help";
    const string help2Arg = "?";
    const string useConfigArg = "useConfig";
    const string tokenHelpArg = "tokenList";
    const string dayIndexArg = "dayIndex";
    const string countryCodeArg = "countryCode";

    #endregion

    private string imageName;
    private string fileName;
    private string filePath;
    private bool setWallPaper = true;
    private WallpaperStyle wallPaperStyle = WallpaperStyle.Stretched;
    private TextBox selectedTextBox = null;
    private Brush fileNameBrush = null;
    private Brush filePathBrush = null;
    private TokenProcessor tokenProcessor = new TokenProcessor();
    private TimeSpan refreshInterval;
    DispatcherTimer dispatcherTimer = new DispatcherTimer();
    
    //private JumpList jumpList = null;

    private int LengthLimit
    {
      get
      {
        // NOTE: Because "Length Limit" is an OS limitation it's stored outside of the BingPaper custom configuration details.
        string lengthLimitTemp = ConfigurationManager.AppSettings["lengthLimit"];
        short limit = defaultLengthLimit;

        if (!string.IsNullOrEmpty(lengthLimitTemp))
          Int16.TryParse(lengthLimitTemp, out limit);

        return limit;
      }
    }

    private BingImageParams GetDefaultParams()
    {
      var imageParams = new BingImageParams();

      imageParams.DayIndex = 0;
      imageParams.Uri = new Uri(defaultRssURL);


      return imageParams;
    }

    private BingPaperConfigSection bingPaperConfig = null;

    private BingPaperConfigSection BingPaperConfig
    {
      get
      {
        if (bingPaperConfig == null)
          LoadSettings();

        return bingPaperConfig;
      }
    }

    const string WelcomeTitle = "BingPaper v1.0 written by: Eric Brown";

    public MainWindow()
    {
      this.InitializeComponent();
      PopulateTokens();
      PopulateCountryCodes();

      if (App.CommandLineArgs != null && App.CommandLineArgs.Count > 0)
      {
        if (ProcessCommandLineArgs())
          Application.Current.Shutdown();
        Hide();
      }
      else
      {
        PopulateRefreshIntervals();
        lengthLabel.Content = Convert.ToString(LengthLimit);
        SetCurrentBingImage(Convert.ToInt16(daySlider.Value));
      }

      SetTitle();

      return;
    }

    public CountryCodes CountryCodes
    {
      get
      {
        return BingUtils.CountryCodes;
      }
      set
      {
        BingUtils.CountryCodes = value;
      }
    }

    private void PopulateRefreshIntervals()
    {
      const int minuteLowerLimit = 5;
      
      int hours = 24;
      int minutes = 59;
      
      List<int> hourList = new List<int>();
      List<int> minuteList = new List<int>();
      List<TimeSpan> timeSpans = new List<TimeSpan>();

      for (int i = hours; i >= 0; i--)
        hourList.Add(i);

      for (int i = minutes; i >= 0; i--)
        minuteList.Add(i);

      foreach (int curHour in hourList)
        foreach (int curMinute in minuteList)
        {
          if (curHour == 0 && curMinute < minuteLowerLimit)
            continue;
          timeSpans.Add(new TimeSpan(curHour, curMinute, 0));
        }

      foreach (TimeSpan curTimeSpan in timeSpans)
        refreshIntervalComboBox.Items.Add(curTimeSpan);
    }
    public static string GenerateHelp()
    {
      StringBuilder sb = new StringBuilder();
      sb.AppendLine(WelcomeTitle);
      sb.AppendLine();
      sb.AppendLine("Useage: ");
      sb.AppendFormat("/{0}:[true/false] - Set the wallpaper to the current Bing image.{1}", setWallPaperArg, Environment.NewLine);
      sb.AppendFormat("/{0}:[save as name] - File name to save the image as.{1}", fileNameArg, Environment.NewLine);
      sb.AppendFormat("/{0}:[path to save file to] - Path to save the image to.{1}  Note: Tokens are valid for file and path values.{1}", filePathArg, Environment.NewLine);
      sb.AppendFormat("/{0} Uses the configuration file settings. (Overrides ALL){1}", useConfigArg, Environment.NewLine);
      sb.AppendFormat("/{0} Displays this help message.{1}", helpArg, Environment.NewLine);
      sb.AppendFormat("/{0} Displays current list of valid tokens.{1}", tokenHelpArg, Environment.NewLine);
      sb.AppendFormat("/{0} [0-7] Is the current day to grab.{1}", dayIndexArg, Environment.NewLine);
      sb.AppendFormat("/{0} [two letter country code] Grabs the image from this country code.{1}", countryCodeArg, Environment.NewLine);
      sb.AppendLine();
      sb.AppendLine("Examples of usage:");
      sb.AppendFormat(@"  BingPaper /{0}:""C:\Pictures\Bing Pics\2009""", fileNameArg);
      sb.AppendLine();
      sb.AppendLine("    Downloads today's Bing image and saves it to the given path.");
      sb.AppendLine();
      sb.AppendFormat(@"  BingPaper /{0} (recommended)", useConfigArg);
      sb.AppendLine();
      sb.AppendLine("    Download's today's Bing image using setting in the config file.");

      return sb.ToString();
    }

    private bool ProcessCommandLineArgs()
    {
      bool result = false;

      if (!IsHelpRequested())
      {
        string useConfigTemp = App.CommandLineArgs[useConfigArg];
        bool useConfig = true;

        GenerateTokenHelp();

        if (!string.IsNullOrEmpty(useConfigTemp))
          useConfig = Convert.ToBoolean(useConfigTemp);

        // NOTE: If useConfig the configuration settings override any parameters passed in.      
        if (useConfig)
        {
          filePath = BingPaperConfig.Settings.Archive.FilePath;
          fileName = BingPaperConfig.Settings.Archive.FileName;
          setWallPaper = BingPaperConfig.Settings.WallPaper.SetWallPaper;
          wallPaperStyle = GetWallPaperStyleFromString(BingPaperConfig.Settings.WallPaper.WallPaperStyle);
        }
        else
        {
          filePath = App.CommandLineArgs[filePathArg];
          fileName = App.CommandLineArgs[fileNameArg];

          string setWallPaperTemp = App.CommandLineArgs[setWallPaperArg];

          if (!string.IsNullOrEmpty(setWallPaperTemp))
            setWallPaper = Convert.ToBoolean(setWallPaperTemp);

          string wallPaperStyleTemp = App.CommandLineArgs[wallPaperStyleArg];

          if (!string.IsNullOrEmpty(wallPaperStyleTemp))
            wallPaperStyle = GetWallPaperStyleFromString(wallPaperStyleTemp);
        }

        result = !string.IsNullOrEmpty(filePath) && !string.IsNullOrEmpty(fileName);

        if (result)
        {
          BingImageParams imageParms = new BingImageParams();

          string countryCode = App.CommandLineArgs[countryCodeArg];

          if (!string.IsNullOrEmpty(countryCode) && countryCode.Length > 2)
          {
            // Assuming the user is passing in the "Long" name of the country rather than the 2 letter code.
            foreach (KeyValuePair<string, string> curItem in BingUtils.CountryCodes)
            {
              if (curItem.Key.Equals(countryCode))
              {
                BingUtils.CurrentCountryCode = curItem.Value;
                break;
              }
            }
          }
          else
            BingUtils.CurrentCountryCode = countryCode;

          
          imageParms.CountryCode = BingUtils.CurrentCountryCode;

          string dayIndex = App.CommandLineArgs[dayIndexArg];

          if (!string.IsNullOrEmpty(dayIndex))
            imageParms.DayIndex = Convert.ToInt16(dayIndex);
          else
            imageParms.DayIndex = 0;

          imageParms.Path = filePath;
          imageParms.File = fileName;
          
          BingImage image = BingUtils.GetBingImage(imageParms);
          string bingImage = image.ImageFileName;

          if (setWallPaper)
            Wallpaper.Set(bingImage, wallPaperStyle);
        }
      }
      else
      {
        MessageBox.Show(GenerateHelp());
        result = true;
      }

      return result;
    }

    private static void GenerateTokenHelp()
    {
      if (!string.IsNullOrEmpty(App.CommandLineArgs[tokenHelpArg]))
      {
        var tokenText = new StringBuilder();
        tokenText.AppendLine("Current valid tokens:");

        foreach (KeyValuePair<string, IReplacementToken> curItem in TokenManager.Tokens)
          tokenText.AppendFormat(string.Format("{0}={1}{2}", curItem.Key, curItem.Value.ReplaceValue, Environment.NewLine));

        MessageBox.Show(tokenText.ToString());
      }
    }

    private static bool IsHelpRequested()
    {
      bool helpRequest = false;

      string displayHelpTemp = App.CommandLineArgs[helpArg];
      string displayHelpTemp2 = App.CommandLineArgs[help2Arg];

      if (!string.IsNullOrEmpty(displayHelpTemp))
        helpRequest = Convert.ToBoolean(displayHelpTemp);

      if (!helpRequest && !string.IsNullOrEmpty(displayHelpTemp2))
        helpRequest = Convert.ToBoolean(displayHelpTemp2);

      return helpRequest;
    }

    private static WallpaperStyle GetWallPaperStyleFromString(string wallPaperStyle)
    {
      return (WallpaperStyle)Enum.Parse(typeof(WallpaperStyle), wallPaperStyle);
    }

    private void LoadSettings()
    {
      bingPaperConfig = ConfigurationManager.GetSection(BingPaperConfigSection.SectionName) as BingPaperConfigSection;

      if (bingPaperConfig != null)
      {
        fileName = bingPaperConfig.Settings.Archive.FileName;
        fileNameTextBox.Text = bingPaperConfig.Settings.Archive.FileName;
        filePath = bingPaperConfig.Settings.Archive.FilePath;
        filePathTextBox.Text = bingPaperConfig.Settings.Archive.FilePath;
        setAsWallPaperCheckBox.IsChecked = bingPaperConfig.Settings.WallPaper.SetWallPaper;
        wallpaperStyle.Text = bingPaperConfig.Settings.WallPaper.WallPaperStyle;
        autoRefreshCheckBox.IsChecked = bingPaperConfig.Settings.AutoRefresh.Enabled;
        refreshInterval = bingPaperConfig.Settings.AutoRefresh.RefreshInterval;
        refreshIntervalComboBox.SelectedItem = bingPaperConfig.Settings.AutoRefresh.RefreshInterval;
        if (!refreshIntervalComboBox.Items.Contains(bingPaperConfig.Settings.AutoRefresh.RefreshInterval))
        {
          refreshIntervalComboBox.Items.Add(refreshInterval);
          refreshIntervalComboBox.SelectedItem = refreshInterval;
        }
      }
      else
        throw new Exception(string.Format("The configuration file is missing the section: {0}.", BingPaperConfigSection.SectionName));
    }

    private void SaveSettings()
    {
      if (IsSaveNeeded())
      {
        MessageBoxResult result = MessageBox.Show("Save settings?", "Question", MessageBoxButton.YesNo, MessageBoxImage.Question);

        if (result == MessageBoxResult.Yes)
        {
          System.Configuration.Configuration bingConfig = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
          BingPaperConfigSection bingConfigSettings = bingConfig.GetSection(BingPaperConfigSection.SectionName) as BingPaperConfigSection;

          bingConfigSettings.Settings.Archive.FileName = fileNameTextBox.Text;
          bingConfigSettings.Settings.Archive.FilePath = filePathTextBox.Text;
          bingConfigSettings.Settings.WallPaper.WallPaperStyle = wallpaperStyle.Text;
          bingConfigSettings.Settings.WallPaper.SetWallPaper = setAsWallPaperCheckBox.IsChecked.Value;
          bingConfigSettings.Settings.AutoRefresh.RefreshInterval = refreshInterval;
          bingConfigSettings.Settings.AutoRefresh.Enabled = autoRefreshCheckBox.IsChecked.Value;

          if (string.IsNullOrEmpty(BingPaperConfig.Settings.BaseURL))
            bingConfigSettings.Settings.BaseURL = defaultBaseURL;

          bingConfig.Save(ConfigurationSaveMode.Modified);
        }
      }
    }

    private void SetCurrentBingImage(int dayIndex)
    {
      string tempFile = IO.Path.GetTempFileName();
      string tempFolder = IO.Path.GetTempPath();
      string fullPath = IO.Path.Combine(tempFolder, tempFile);
     
      BingImage currentBingImage = BingUtils.GetBingImage(BingUtils.CurrentCountryCode, dayIndex);

      //if (BingPaperConfig != null)
      //  tempFile = BingUtils.GetCurrentBingImage(new Uri(BingPaperConfig.Settings.BaseURL), tempFolder);

      if (!string.IsNullOrEmpty(tempFile))
        bingImage.Source = new BitmapImage(currentBingImage.ImageURL);
    }

    private void PopulateTokens()
    {
      BingUtils.LoadTokens();
      
      tokenListBox.Items.Clear();

      foreach (KeyValuePair<string, IReplacementToken> curItem in TokenManager.Tokens)
        tokenListBox.Items.Add(string.Format("{0}={1}", curItem.Key, curItem.Value.ReplaceValue));
    }

    private void PopulateCountryCodes()
    {
      countryCodesComboBox.Items.Clear();

      foreach (KeyValuePair<string, string> curItem in BingUtils.CountryCodes)
        countryCodesComboBox.Items.Add(curItem.Key);
    }

    private void Button_Click(object sender, System.Windows.RoutedEventArgs e)
    {
      GetImageAndSetWallpaper();
    }

    private bool IsSaveNeeded()
    {
      bool saveNeeded = false;

      filePathTextBox.Text = StringUtils.AppendPathDelimeter(filePathTextBox.Text);

      saveNeeded = BingPaperConfig.Settings.Archive.FilePath != filePathTextBox.Text;

      if (!saveNeeded)
        saveNeeded = BingPaperConfig.Settings.Archive.FileName != fileNameTextBox.Text;

      if (!saveNeeded)
        saveNeeded = BingPaperConfig.Settings.WallPaper.SetWallPaper != setAsWallPaperCheckBox.IsChecked;

      if (!saveNeeded)
        saveNeeded = BingPaperConfig.Settings.AutoRefresh.RefreshInterval != refreshInterval;

      if (!saveNeeded)
        saveNeeded = BingPaperConfig.Settings.AutoRefresh.Enabled != autoRefreshCheckBox.IsChecked.Value;

      return saveNeeded;
    }

    private BingImage GetCurrentBingImage()
    {
      var bingImage = new BingImage();

      bingImage = BingUtils.GetBingImage(BingUtils.CurrentCountryCode, Convert.ToInt16(daySlider.Value));

      return bingImage;
    }

    private BingImage GetCurrentBingImage(BingImageParams imageParams)
    {
      var bingImage = new BingImage();

      if (imageParams != null && imageParams.Validate())
        bingImage = BingUtils.GetBingImage(imageParams);

      return bingImage;
    }

    private void GetImageAndSetWallpaper()
    {
      imageName = BingPaperConfig.Settings.Archive.FileName;
      fileName = string.Empty;

      string baseURL = BingPaperConfig.Settings.BaseURL;

      if (!filePathTextBox.Text.EndsWith(@"\"))
        filePathTextBox.Text = filePathTextBox.Text + @"\";

      if (BingPaperConfig.Settings.Archive.ArchiveImage)
      {
        if (string.IsNullOrEmpty(baseURL))
          baseURL = defaultBaseURL;

        BingImageParams imageParams = new BingImageParams();
        imageParams.DayIndex = Convert.ToInt16(daySlider.Value);
        imageParams.CountryCode = BingUtils.CurrentCountryCode;
        
        if (!string.IsNullOrEmpty(filePathTextBox.Text))
          imageParams.Path = filePathTextBox.Text;

        if (!string.IsNullOrEmpty(imageName))
          imageParams.File = fileNameTextBox.Text;
        
        var image = BingUtils.GetBingImage(imageParams);

        fileName = image.ImageFileName;

        var tempPath = IO.Path.GetTempFileName();
        var tempFile = IO.Path.GetTempFileName();
        var tempImage = IO.Path.Combine(tempPath, tempFile);

        if (IO.File.Exists(tempImage))
          IO.File.Delete(tempImage);

        if (!string.IsNullOrEmpty(fileName))
          IO.File.Copy(fileName, tempFile);
        
        if (!string.IsNullOrEmpty(tempImage))
        {
          bingImage.Source = new BitmapImage(new Uri(tempImage));

          if (setAsWallPaperCheckBox.IsChecked.HasValue && setAsWallPaperCheckBox.IsChecked.Value == true)
          {
            WallpaperStyle style = (WallpaperStyle)Enum.Parse(typeof(WallpaperStyle), wallpaperStyle.Text, true);
            Wallpaper.Set(new Uri(fileName), style);
          }
        }
      }
    }

    private string GetPath()
    {
      string parsedFilePath = BingUtils.ParseTokens(filePathTextBox.Text);
      string parsedFileName = BingUtils.ParseTokens(fileNameTextBox.Text);

      return IO.Path.Combine(parsedFilePath, parsedFileName);
    }

    private void fileNameTextBox_TextChanged(object sender, TextChangedEventArgs e)
    {
      var path = GetPath();

      ToggleTextFields(path);

      SetResultLabel();
    }

    private void ToggleTextFields(string path)
    {
      int total = LengthLimit - path.Length;

      if (total <= 0)
      {
        filePathTextBox.Background = Brushes.DarkRed;
        fileNameTextBox.Background = Brushes.DarkRed;
        goButton.IsEnabled = false;
        dispatcherTimer.IsEnabled = false;
      }
      else
      {
        goButton.IsEnabled = true;
        filePathTextBox.Background = filePathBrush;
        fileNameTextBox.Background = fileNameBrush;
        dispatcherTimer.IsEnabled = true;
      }
    }

    private void SetResultLabel()
    {
      var path = GetPath();
      resultantPathLabel.Content = path;
      lengthLabel.Content = LengthLimit - path.Length;
    }

    private void tokenListBox_MouseDoubleClick(object sender, MouseButtonEventArgs e)
    {
      if (sender is ListBox)
      {
        var tokenSource = sender as ListBox;
        int index = tokenListBox.SelectedItem.ToString().IndexOf('=');
        string selectedToken = tokenListBox.SelectedItem.ToString().Substring(0, index);

        if (selectedTextBox != null)
        {
          selectedTextBox.Text = selectedTextBox.Text + selectedToken;
          selectedTextBox.Focus();
          selectedTextBox.CaretIndex = selectedTextBox.Text.Length + 1;
        }
      }
    }

    private void fileNameTextBox_LostFocus(object sender, RoutedEventArgs e)
    {
      selectedTextBox = fileNameTextBox;
    }

    private void filePathTextBox_LostFocus(object sender, RoutedEventArgs e)
    {
      selectedTextBox = filePathTextBox;
    }

    private void filePathTextBox_Loaded(object sender, RoutedEventArgs e)
    {
      filePathBrush = filePathTextBox.Background;
      fileNameBrush = fileNameTextBox.Background;
    }

    private void Window_Loaded(object sender, RoutedEventArgs e)
    {
      LoadSettings();

      if (autoRefreshCheckBox.IsChecked.Value)
      {
        dispatcherTimer.Tick += new EventHandler(dispatcherTimer_Tick);
        dispatcherTimer.Interval = refreshInterval;

        SetRefreshLabel();

        dispatcherTimer.Start();
        
        refreshCountdownLabel.Visibility = System.Windows.Visibility.Visible;
      }

      countryCodesComboBox.SelectedValue = BingUtils.CountryCodes.DefaultCountryCodeKey;

      refreshIntervalComboBox.IsEnabled = autoRefreshCheckBox.IsChecked.Value;
    }

    private void SetRefreshLabel()
    {
      DateTime nextRefresh = DateTime.Now.Add(refreshInterval);
      refreshCountdownLabel.Content = string.Format("Next Refresh At: {0}", nextRefresh.ToString("hh:mm:ss"));

      if (autoRefreshCheckBox.IsChecked.Value)
        refreshCountdownLabel.Visibility = System.Windows.Visibility.Visible;
      else
        refreshCountdownLabel.Visibility = System.Windows.Visibility.Hidden;
    }
    
    private void dispatcherTimer_Tick(object sender, EventArgs e)
    {
      if (autoRefreshCheckBox.IsChecked.Value)
      {
        PopulateTokens();
        SetResultLabel();
        GetImageAndSetWallpaper();
      }
      else
        dispatcherTimer.Stop();

      SetRefreshLabel();
    }

    private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
    {
      SaveSettings();
    }

    private void autoRefreshCheckBox_Click(object sender, RoutedEventArgs e)
    {
      refreshIntervalComboBox.IsEnabled = autoRefreshCheckBox.IsChecked.Value;
      if (!autoRefreshCheckBox.IsChecked.Value)
      {
        dispatcherTimer.Stop();
        refreshCountdownLabel.Visibility = Visibility.Hidden;
      }
      else
      {
        dispatcherTimer.Start();
        SetRefreshLabel();
        refreshCountdownLabel.Visibility = Visibility.Visible;
      }
    }

    private void refreshIntervalComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      if (refreshIntervalComboBox.SelectedItem is TimeSpan)
      {
        dispatcherTimer.Interval = (TimeSpan)refreshIntervalComboBox.SelectedItem;
        refreshInterval = dispatcherTimer.Interval;
        SetRefreshLabel();
      }
    }

    private void countryCodesComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      if (e.AddedItems.Count > 0)
      {
        string selectedCountryKey = e.AddedItems[0].ToString();
        BingUtils.CurrentCountryCode = BingUtils.CountryCodes[selectedCountryKey];
        PopulateTokens();
        SetResultLabel();
        SetCurrentBingImage(Convert.ToInt16(daySlider.Value));
        SetTitle(BingUtils.CurrentCountryCode);
      }
    }

    private void SetTitle(string countryCode = null)
    {
      if (!string.IsNullOrEmpty(countryCode))
        Title = string.Format(appTitleWithCountryCode, countryCode);
      else
        Title = appTitle;
    }

    private void daySlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
    {
      SetCurrentBingImage(Convert.ToInt16(e.NewValue));
      PopulateTokens();
      SetResultLabel();
    }
  }
}