﻿#region codejockey.net Copyright Banner
/***************************************************************\
 * Copyright 2010 codejockey.net.                                *
 *                                                               *
 * This code is property of codejockey.net and cannot be used    *
 * without explicit permission given by a codejockey.net         *
 * Representative.  For details contact: source@codejockey.net   *
 \***************************************************************/
#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Security.Principal;
using System.Xml;
using System.Net;
using System.Drawing;

using codejockey.Shared.Web;
using codejockey.Shared.XML;
using codejockey.Shared.Win32;
using codejockey.Shared.Text.ReplacementTokens;

using codejockey.BingPaper.Utils.ReplacementTokens;

namespace codejockey.BingPaper.Utils
{
  public static class BingUtils
  {
    #region Default Values

    private const string TrackerFileName = "BingDLStatus{0}.xml";
    private const string baseURL = "http://www.bing.com";
    private const string RssURLWithCountryCodeAndIndexParams = baseURL + "/HPImageArchive.aspx?format=xml&idx={0}&n=1&cc={1}";
    private const string defaultFileName = "$month$$day$$year$.jpg";
    
    #endregion

    #region XML Nodes

    private const string imageNodeName = "images/image";
    private const string messagesNodeName = "images/image/messages/message";
    private const string messageTextNodeName = "msgtext";
    private const string messageTitleNodeName = "msgtitle";
    private const string messageLinkNodeName = "msglink";
    private const string urlNodeName = "url";
    private const string copyrightNodeName = "copyright";

    private const string hotSpotNodeName = "images/image/hotspots/hotspot";
    private const string hotSpotDescriptionNodeName = "desc";
    private const string hotSpotLinkNodeName = "link";
    private const string hotSpotQueryNodeName = "query";
    private const string hotSpotLocXNodeName = "LocX";
    private const string hotSpotLocYNodeName = "LocY";

    #endregion

    #region Messages

    private const string imageDownloadedMessage = "Image: {0} has already been downloaded.";
    private const string imageNewlyDownloadedMessage = "Image: {0} newly downloaded.";
    private const string unableToFindImageNodeErrorMessage = "Unable to find Image node";
    private const string unableToFindURLNodeMessage = "Unable to find URL node";
    private const string unableToContactWebServerMessage = "Unable to contact the web server.";
    private const string validUrlObjectMissingMessage = "A valid Uri object must be provided.";
    private const string validPathMissingMessage = "A valid path must be provided to save the file to.";

    #endregion

    private static string bingImageName = string.Empty;
    private static string currentCountryCode = string.Empty;
    private static CountryCodes countryCodes = new CountryCodes();

    static BingUtils()
    {
      LoadTokens();
      AddCustomTokens();
    }

    private static void AddCustomTokens()
    {
      var bingImageToken = new BingImageNameReplacementToken(ScrubBingString(GetBingImageName()));
      var countryCodeToken = new CountryCodeReplacementToken();

      if (!TokenManager.Tokens.ContainsKey(bingImageToken.Token))
        TokenManager.Tokens.Add(bingImageToken.Token, bingImageToken);

      if (!TokenManager.Tokens.ContainsKey(countryCodeToken.Token))
        TokenManager.Tokens.Add(countryCodeToken.Token, countryCodeToken);
    }

    /// <summary>
    /// Forces the download by deleting the tracking file used to determine if BingPaper should download again.
    /// </summary>
    public static void ForceDownload()
    {
      string trackerFileCountryCode = string.Format(TrackerFileName, BingUtils.CurrentCountryCode);
      string trackerFile = string.Format(@"{0}\codejockey.net\BingPaper\{1}", Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), trackerFileCountryCode);

      if (File.Exists(trackerFile))
        File.Delete(trackerFile);
    }

    /// <summary>
    /// Determines whether [is download needed] [the specified image name].
    /// </summary>
    /// <param name="imageName">Name of the image.</param>
    /// <param name="fileName">Name of the file.</param>
    /// <returns>
    /// 	<c>true</c> if [is download needed] [the specified image name]; otherwise, <c>false</c>.
    /// </returns>
    private static bool IsDownloadNeeded(string imageName, string fileName)
    {
      bool result = false;
      LastImageRetrieved imageStatus = new LastImageRetrieved();

      string tempTrackerFile = string.Format(TrackerFileName, BingUtils.CurrentCountryCode);
      string trackerFile = string.Format(@"{0}\\codejockey.net\BingPaper\{1}", Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), tempTrackerFile);

      if (File.Exists(trackerFile))
        imageStatus = XMLUtils.DeserializeXmlFromFile(trackerFile, typeof(LastImageRetrieved)) as LastImageRetrieved;

      string remoteImage = ScrubBingString(imageName);
      string lastImage = string.Empty;
      imageStatus.LastCheck = DateTime.Now;

      if (!string.IsNullOrEmpty(imageStatus.ImageName))
        lastImage = ScrubBingString(imageStatus.ImageName);

      if (remoteImage.CompareTo(lastImage) == 0)
      {
        imageStatus.LastStatus = string.Format(imageDownloadedMessage, imageName);
        result = false;
      }
      else
      {
        result = true;
        imageStatus = new LastImageRetrieved
        {
          ImageName = imageName,
          LastStatus = string.Format(imageNewlyDownloadedMessage, imageName),
          Retrieval = DateTime.Now,
        };
      }

      XMLUtils.SerializeXMLToFile(trackerFile, imageStatus);

      return result;
    }


    public static void LoadTokens()
    {
      TokenManager.LoadTokens();
      AddCustomTokens();
    }

    public static string CurrentCountryCode
    {
      get
      {
        if (string.IsNullOrEmpty(currentCountryCode))
          currentCountryCode = countryCodes[BingUtils.CountryCodes.DefaultCountryCodeKey];
        
        return currentCountryCode;
      }
      set
      {
        currentCountryCode = value;
      }
    }

    public static int CurrentDayIndex
    {
      get;
      set;
    }

    public static CountryCodes CountryCodes
    {
      get
      {
        return countryCodes;
      }

      set
      {
        countryCodes = value;
      }
    }

    public static BingImage GetBingImage(BingImageParams bingImageParams)
    {
      var imageDetails = new BingImage();

      if (bingImageParams != null)
      {
        if (bingImageParams.Validate())
        {
          HttpWebRequest request = (HttpWebRequest)WebRequest.Create(bingImageParams.Uri);
          HttpWebResponse response = (HttpWebResponse)request.GetResponse();
          Stream responseStream = response.GetResponseStream();
          XmlDocument doc = new XmlDocument();
          doc.Load(responseStream);

          //Grab the frist node out of the list, as currently the RSS only contains one image.      
          XmlNode imageNode = doc.SelectSingleNode(imageNodeName);
          if (imageNode == null)
            throw new Exception(unableToFindImageNodeErrorMessage);

          XmlNode urlNode = imageNode.SelectSingleNode(urlNodeName);

          string fileName = ScrubBingString(urlNode.InnerText);
          imageDetails.ImageName = fileName;

          imageDetails.Copyright = imageNode.SelectSingleNode(copyrightNodeName).InnerText;
          imageDetails.ImageURL = new Uri(string.Format("{0}{1}", baseURL, urlNode.InnerText));

          imageDetails.CurrentImage = GetCurrentImage(imageDetails.ImageURL, bingImageParams.Path, bingImageParams.File);

          if (!string.IsNullOrEmpty(bingImageParams.Path) && !string.IsNullOrEmpty(bingImageParams.File))
            imageDetails.ImageFileName = Path.Combine(TokenManager.Parse(bingImageParams.Path), TokenManager.Parse(bingImageParams.File));
          
          if (urlNode == null)
            throw new Exception(unableToFindURLNodeMessage);

          imageDetails.HotSpots = GetHotspots(doc);
          imageDetails.Messsages = GetMessages(doc);

          if (!string.IsNullOrEmpty(bingImageParams.CountryCode))
            CurrentCountryCode = bingImageParams.CountryCode;

          CurrentDayIndex = bingImageParams.DayIndex;
        }
      }
      return imageDetails;
    }

    public static BingImage GetBingImage(string countryCode)
    {
      BingImageParams BingParams = new BingImageParams();

      BingParams.CountryCode = countryCode;

      return GetBingImage(BingParams);
    }

    public static BingImage GetBingImage(string countryCode, int dayIndex)
    {
      BingImageParams BingParams = new BingImageParams();

      BingParams.CountryCode = countryCode;
      BingParams.DayIndex = dayIndex;

      return GetBingImage(BingParams);
    }

    public static BingImage GetBingImage(string countryCode, int dayIndex, string fileName)
    {
      BingImageParams BingParams = new BingImageParams();

      BingParams.CountryCode = countryCode;
      BingParams.DayIndex = dayIndex;
      BingParams.File = fileName;

      return GetBingImage(BingParams);

    }

    private static Image GetCurrentImage(Uri uri, string path, string fileName)
    {
      bool temp = false;

      if (string.IsNullOrEmpty(path))
      {
        path = Path.GetTempPath();
        temp = true;
      }

      if (string.IsNullOrEmpty(fileName))
      {
        fileName = Path.GetTempFileName();
        temp = true;
      }

      string tempImage = Path.Combine(TokenManager.Parse(path), TokenManager.Parse(fileName));

      CreateFilePath(TokenManager.Parse(path));
      Image result = null;

      WebUtils.GetFile(uri, tempImage);

      using (Stream s = new WebClient().OpenRead(tempImage))
      {
        result = Image.FromStream(s);
        s.Flush();
        s.Close();
      }

      if (temp)
        File.Delete(tempImage);

      return result;
    }

    private static List<Message> GetMessages(XmlDocument doc)
    {
      // Grab the messages node.
      XmlNodeList messages = doc.SelectNodes(messagesNodeName);
      var bingMessages = new List<Message>();

      if (messages != null)
      {
        for (int i = 0; i < messages.Count; i++)
        {
          Message message = new Message();
          message.Text = messages[i].SelectSingleNode(messageTextNodeName).InnerText;
          message.Title = messages[i].SelectSingleNode(messageTitleNodeName).InnerText;
          message.Link = new Uri(messages[i].SelectSingleNode(messageLinkNodeName).InnerText);

          bingMessages.Add(message);
        }
      }

      return bingMessages;
    }

    private static List<HotSpot> GetHotspots(XmlDocument doc)
    {
      // Grab the hotspots node.
      XmlNodeList hotSpots = doc.SelectNodes(hotSpotNodeName);
      var bingHotSpots = new List<HotSpot>();

      if (hotSpots != null)
      {
        for (int i = 0; i < hotSpots.Count; i++)
        {
          HotSpot hotSpot = new HotSpot();
          hotSpot.Description = hotSpots[i].SelectSingleNode(hotSpotDescriptionNodeName).InnerText;
          hotSpot.Link = new Uri(hotSpots[i].SelectSingleNode(hotSpotLinkNodeName).InnerText);
          hotSpot.Query = hotSpots[i].SelectSingleNode(hotSpotQueryNodeName).InnerText;

          int locX;
          if (!int.TryParse(hotSpots[i].SelectSingleNode(hotSpotLocXNodeName).InnerText, out locX))
            hotSpot.LocationX = 0;
          else
            hotSpot.LocationX = locX;

          int locY;
          if (!int.TryParse(hotSpots[i].SelectSingleNode(hotSpotLocYNodeName).InnerText, out locY))
            hotSpot.LocationY = 0;
          else
            hotSpot.LocationY = locY;

          bingHotSpots.Add(hotSpot);
        }
      }

      return bingHotSpots;
    }

    private static string GetBingImageName()
    {
      var rssURL = string.Format(RssURLWithCountryCodeAndIndexParams, CurrentDayIndex, CurrentCountryCode);

      HttpWebRequest request = (HttpWebRequest)WebRequest.Create(rssURL);
      HttpWebResponse response = null;
      try
      {
        response = (HttpWebResponse)request.GetResponse();
      }
      catch
      {
        throw new Exception(unableToContactWebServerMessage);
      }
      Stream responseStream = response.GetResponseStream();
      XmlDocument doc = new XmlDocument();
      doc.Load(responseStream);

      //Grab the frist node out of the list, as currently the RSS only 
      //  contains one image.      
      XmlNode imageNode = doc.SelectSingleNode(imageNodeName);
      if (imageNode == null)
        throw new Exception(unableToFindImageNodeErrorMessage);

      XmlNode urlNode = imageNode.SelectSingleNode(urlNodeName);

      if (urlNode == null)
        throw new Exception(unableToFindURLNodeMessage);

      var nodeText = urlNode.InnerText;

      return nodeText;
    }
    
    private static string ScrubBingString(string sourceString)
    {
      int index = sourceString.LastIndexOf("/");
      bingImageName = sourceString.Substring(index + 1);
      bingImageName = bingImageName.Replace(".jpg", "");

      return bingImageName;
    }

    /// <summary>
    /// Parses the name of the tokens from path.
    /// </summary>
    /// <param name="tokenizedString">The path to save file to.</param>
    /// <returns>A string with known tokens replaced.</returns>
    public static string ParseTokens(string tokenizedString)
    {
      return TokenManager.Parse(tokenizedString);
    }

    private static void CreateFilePath(string pathToSaveFileTo)
    {
      if (!Directory.Exists(pathToSaveFileTo))
        Directory.CreateDirectory(pathToSaveFileTo);
    }

    private static string GetDefaultFileName()
    {
      return TokenManager.Parse(defaultFileName);
    }

    private static void ValidateParams(Uri url, string pathToSaveFileTo)
    {
      if (url == null)
        throw new Exception(validUrlObjectMissingMessage);

      if (string.IsNullOrEmpty(pathToSaveFileTo))
        throw new Exception(validPathMissingMessage);
    }
  }
}
